@@ -5,40 +5,65 @@
  */
 
 #include "fd.h"
+#include "time.h"
+#include "log.h"
+#include "termio.h"
+#include "realpath.h"
+#include "parse.h"
+#include "kconv.h"
 #include "func.h"
 #include "kanji.h"
 #include "version.h"
 
+#ifdef	DEP_ORIGSHELL
+#include "system.h"
+#endif
+#ifdef	DEP_PTY
+#include "termemu.h"
+#endif
+#ifdef	DEP_IME
+#include "roman.h"
+#endif
+#ifdef	DEP_URLPATH
+#include "urldisk.h"
+#endif
+
 #if	MSDOS
 #include <process.h>
 # ifndef	DJGPP
 # include <dos.h>
-#  ifdef	__TURBOC__
-extern u_int _stklen = 0x5800;
-#  define	harderr_t	void
-#  else
-#  define	harderr_t	int
-#  endif
 # endif
 #endif	/* !MSDOS */
 
-#ifdef	_NOORIGSHELL
-#include "termio.h"
-#include "wait.h"
-#define	Xexit2		exit2
-#define	isorgpid()	(1)
-#else
-#include "system.h"
-#define	isorgpid()	(mypid == orgpid)
+#define	CLOCKUPDATE		10	/* sec */
+#define	SWITCHUSER		"su"
+#ifndef	BINDIR
+#define	BINDIR			"/usr/local/bin"
 #endif
-
-#if	!defined (_NOIME) && defined (DEBUG)
-#include "roman.h"
-#endif
-
 #ifndef	DATADIR
-#define	DATADIR		progpath
+#define	DATADIR			progpath
 #endif
+#ifndef	DEFPATH
+#define	DEFPATH			":/bin:/usr/bin"
+#endif
+
+#ifdef	DEP_ORIGSHELL
+#define	isorgpid()		(mypid == orgpid)
+#else
+#define	isorgpid()		(1)
+#endif
+
+#ifdef	__TURBOC__
+extern u_int _stklen = 0x5800;
+#define	harderr_t		void
+#else
+#define	harderr_t		int
+#endif
+
+typedef struct _sigfunc_t {
+	int sig;
+	int (*func)__P_((VOID_A));
+} sigfunc_t;
 
 #if	MSDOS
 # ifndef	BSPATHDELIM
@@ -49,25 +74,39 @@ extern char *adjustfname __P_((char *));
 # endif
 #endif	/* MSDOS */
 
-#if	!defined (_NOORIGSHELL) && !defined (NOJOB)
+#if	defined (DEP_ORIGSHELL) && !defined (NOJOB)
 extern VOID killjob __P_((VOID_A));
 #endif
 
-#ifdef	_NOORIGSHELL
+#ifndef	DEP_ORIGSHELL
 extern char **environ;
 extern char **environ2;
 #endif
-extern bindtable bindlist[];
+extern bindlist_t bindlist;
+extern int maxbind;
+#if	defined (DEP_DYNAMICLIST) || !defined (_NOCUSTOMIZE)
+extern origbindlist_t origbindlist;
+extern int origmaxbind;
+#endif
 #ifndef	_NOARCHIVE
-extern launchtable launchlist[];
+extern launchlist_t launchlist;
 extern int maxlaunch;
-extern archivetable archivelist[];
+extern archivelist_t archivelist;
 extern int maxarchive;
+# if	defined (DEP_DYNAMICLIST) || !defined (_NOCUSTOMIZE)
+extern origlaunchlist_t origlaunchlist;
+extern int origmaxlaunch;
+extern origarchivelist_t origarchivelist;
+extern int origmaxarchive;
+# endif
+#endif	/* !_NOARCHIVE */
+extern helpindex_t helpindex;
+#if	defined (DEP_DYNAMICLIST) || !defined (_NOCUSTOMIZE)
+extern orighelpindex_t orighelpindex;
 #endif
 extern int wheader;
 extern char fullpath[];
 extern short savehist[];
-extern char *helpindex[];
 extern int subwindow;
 extern int win_x;
 extern int win_y;
@@ -75,23 +114,16 @@ extern int win_y;
 extern int custno;
 #endif
 extern char *deftmpdir;
-#ifdef	_USEUNICODE
-extern char *unitblpath;
-#endif
-#ifndef	_NOIME
+#ifdef	DEP_IME
 extern char *dicttblpath;
 #endif
-#ifndef	_NOPTY
-extern int ptymode;
-extern int ptyinternal;
-extern int parentfd;
-#endif
-
-#define	CLOCKUPDATE	10	/* sec */
 
 #if	MSDOS && !defined (PROTECTED_MODE)
 static harderr_t far criticalerror __P_((u_int, u_int, u_short far *));
 #endif
+#ifndef	DEP_ORIGSHELL
+static VOID NEAR Xexit2 __P_((int));
+#endif
 static VOID NEAR signalexit __P_((int));
 #ifdef	SIGALRM
 static int ignore_alrm __P_((VOID_A));
@@ -158,23 +190,26 @@ static VOID NEAR printtime __P_((int));
 #ifdef	SIGALRM
 static int trapalrm __P_((VOID_A));
 #endif
-#ifndef	_NODOSDRIVE
-static int wrap_intrkey __P_((VOID_A));
+#ifdef	DEP_DOSDRIVE
+static int dos_intrkey __P_((VOID_A));
 #endif
-static char *NEAR getversion __P_((int *));
-#ifndef	_NOLOGGING
-static VOID NEAR startlog __P_((char **));
+static int read_intrkey __P_((VOID_A));
+#ifdef	DEP_LOGGING
+static VOID NEAR startlog __P_((char *CONST *));
 static VOID NEAR endlog __P_((int));
 #endif
-#ifdef	_NOORIGSHELL
-static int NEAR execruncomline __P_((char *, char *, int, char *));
+#ifndef	DEP_ORIGSHELL
+static int NEAR execruncomline __P_((char *, CONST char *, int, CONST char *));
 #endif
-static int NEAR initoption __P_((int, char *[], char *[]));
-static int NEAR evaloption __P_((char *[]));
-static char *NEAR searchenv __P_((char *, char *[]));
-static VOID NEAR setexecname __P_((char *));
-static VOID NEAR setexecpath __P_((char *, char *[]));
-int main __P_((int, char *[], char *[]));
+static int NEAR initoption __P_((int, char *CONST []));
+static int NEAR evaloption __P_((char *CONST []));
+static char *NEAR searchenv __P_((CONST char *, char *CONST []));
+#if	!MSDOS
+static char *NEAR searchexecname __P_((CONST char *, char *CONST []));
+#endif
+static VOID NEAR setexecname __P_((CONST char *));
+static VOID NEAR setexecpath __P_((CONST char *, char *CONST []));
+int main __P_((int, char *CONST [], char *CONST []));
 
 char *origpath = NULL;
 char *progpath = NULL;
@@ -184,32 +219,16 @@ char *tmpfilename = NULL;
 int adjtty = 0;
 #endif
 int showsecond = 0;
-#if	FD >= 2
-int thruargs = 0;
-#endif
 int hideclock = 0;
 #ifdef	SIGALRM
 int noalrm = 1;
 #endif
 int fd_restricted = 0;
-#ifndef	_NOCUSTOMIZE
-char **orighelpindex = NULL;
-bindtable *origbindlist = NULL;
-# ifndef	_NOKEYMAP
+#if	!defined (_NOCUSTOMIZE) && !defined (_NOKEYMAP)
 keyseq_t *origkeymaplist = NULL;
-# endif
-# ifndef	_NOARCHIVE
-launchtable *origlaunchlist = NULL;
-int origmaxlaunch = 0;
-archivetable *origarchivelist = NULL;
-int origmaxarchive = 0;
-# endif
-# ifdef	_USEDOSEMU
-devinfo *origfdtype = NULL;
-# endif
-#endif	/* !_NOCUSTOMIZE */
+#endif
 int inruncom = 0;
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 int fdmode = 0;
 #endif
 
@@ -217,6 +236,57 @@ static int timersec = 0;
 #ifdef	SIGWINCH
 static int nowinch = 0;
 #endif
+static CONST sigfunc_t sigfunclist[] = {
+#ifdef	SIGINT
+	{SIGINT, ignore_int},
+#endif
+#ifdef	SIGQUIT
+	{SIGQUIT, ignore_quit},
+#endif
+#ifdef	SIGCONT
+	{SIGCONT, ignore_cont},
+#endif
+#ifdef	SIGHUP
+	{SIGHUP, hanguperror},
+#endif
+#ifdef	SIGILL
+	{SIGILL, illerror},
+#endif
+#ifdef	SIGTRAP
+	{SIGTRAP, traperror},
+#endif
+#ifdef	SIGIOT
+	{SIGIOT, ioerror},
+#endif
+#ifdef	SIGEMT
+	{SIGEMT, emuerror},
+#endif
+#ifdef	SIGFPE
+	{SIGFPE, floaterror},
+#endif
+#ifdef	SIGBUS
+	{SIGBUS, buserror},
+#endif
+#ifdef	SIGSEGV
+	{SIGSEGV, segerror},
+#endif
+#ifdef	SIGSYS
+	{SIGSYS, syserror},
+#endif
+#ifdef	SIGPIPE
+	{SIGPIPE, pipeerror},
+#endif
+#ifdef	SIGTERM
+	{SIGTERM, terminate},
+#endif
+#ifdef	SIGXCPU
+	{SIGXCPU, xcpuerror},
+#endif
+#ifdef	SIGXFSZ
+	{SIGXFSZ, xsizerror},
+#endif
+	{-1, NULL}
+};
 
 
 #if	MSDOS && !defined (PROTECTED_MODE)
@@ -231,42 +301,51 @@ u_short far *devhdr;
 #endif
 
 VOID error(s)
-char *s;
+CONST char *s;
 {
+	static int doing = 0;
 	int duperrno;
 
+	if (doing == 1) exit2(2);
+	if (doing == 2) exit(2);
 	duperrno = errno;
+	doing = 1;
+
 	sigvecset(0);
 	if (isorgpid()) {
 		forcecleandir(deftmpdir, tmpfilename);
-#ifndef	_NODOSDRIVE
-		dosallclose();
+#ifdef	DEP_DOSDRIVE
+		VOID_C dosallclose();
+#endif
+#ifdef	DEP_URLPATH
+		VOID_C urlallclose();
 #endif
 		Xstdiomode();
 		endterm();
 	}
 
 	if (!s) s = progname;
-	if (dumbterm <= 2) fputc('\007', stderr);
+	if (dumbterm <= 2) Xfputc('\007', Xstderr);
 	errno = duperrno;
 	if (errno) perror2(s);
 	else {
-		fputs(s, stderr);
-		fputnl(stderr);
+		Xfputs(s, Xstderr);
+		VOID_C fputnl(Xstderr);
 	}
-	fclose(stderr);
+	VOID_C Xfclose(Xstderr);
+	doing = 2;
 
 	if (isorgpid()) {
 		inittty(1);
 		keyflush();
 		prepareexitfd(2);
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 # ifndef	NOJOB
 		killjob();
 # endif
 		prepareexit(0);
-#endif	/* !_NOORIGSHELL */
-#ifndef	_NOPTY
+#endif	/* DEP_ORIGSHELL */
+#ifdef	DEP_PTY
 		killallpty();
 #endif
 #ifdef	DEBUG
@@ -281,29 +360,42 @@ char *s;
 	exit(2);
 }
 
+#ifndef	DEP_ORIGSHELL
+static VOID NEAR Xexit2(n)
+int n;
+{
+	prepareexitfd(n);
+	exit2(n);
+}
+#endif	/* !DEP_ORIGSHELL */
+
 static VOID NEAR signalexit(sig)
 int sig;
 {
-	signal2(sig, SIG_IGN);
+	VOID_C signal2(sig, SIG_IGN);
 
 	if (isorgpid()) {
+		printf_urgent++;
 		forcecleandir(deftmpdir, tmpfilename);
-#ifndef	_NODOSDRIVE
-		dosallclose();
+#ifdef	DEP_DOSDRIVE
+		VOID_C dosallclose();
 #endif
-#ifndef	_NOLOGGING
+#ifdef	DEP_URLPATH
+		VOID_C urlallclose();
+#endif
+#ifdef	DEP_LOGGING
 		endlog(sig + 128);
 #endif
 		endterm();
 		inittty(1);
 		keyflush();
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 # if	defined (SIGHUP) && !defined (NOJOB)
 		if (sig == SIGHUP) killjob();
 # endif
 		prepareexit(0);
-#endif	/* !_NOORIGSHELL */
-#ifndef	_NOPTY
+#endif	/* DEP_ORIGSHELL */
+#ifdef	DEP_PTY
 		killallpty();
 #endif
 #ifdef	DEBUG
@@ -313,9 +405,10 @@ int sig;
 		closetty(&ttyio, &ttyout);
 		muntrace();
 #endif	/* DEBUG */
+		printf_urgent--;
 	}
 
-	signal2(sig, SIG_DFL);
+	VOID_C signal2(sig, SIG_DFL);
 	VOID_C kill(getpid(), sig);
 }
 
@@ -325,7 +418,7 @@ static int ignore_alrm(VOID_A)
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGALRM, (sigcst_t)ignore_alrm);
+	VOID_C signal2(SIGALRM, (sigcst_t)ignore_alrm);
 	errno = duperrno;
 
 	return(0);
@@ -338,7 +431,7 @@ static int ignore_winch(VOID_A)
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGWINCH, (sigcst_t)ignore_winch);
+	VOID_C signal2(SIGWINCH, (sigcst_t)ignore_winch);
 	errno = duperrno;
 
 	return(0);
@@ -351,7 +444,7 @@ static int ignore_int(VOID_A)
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGINT, (sigcst_t)ignore_int);
+	VOID_C signal2(SIGINT, (sigcst_t)ignore_int);
 	errno = duperrno;
 
 	return(0);
@@ -364,7 +457,7 @@ static int ignore_quit(VOID_A)
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGQUIT, (sigcst_t)ignore_quit);
+	VOID_C signal2(SIGQUIT, (sigcst_t)ignore_quit);
 	errno = duperrno;
 
 	return(0);
@@ -377,10 +470,10 @@ static int ignore_cont(VOID_A)
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGCONT, (sigcst_t)ignore_cont);
+	VOID_C signal2(SIGCONT, (sigcst_t)ignore_cont);
 # if	!MSDOS
 	suspended = 1;
-#  ifndef	_NOORIGSHELL
+#  ifdef	DEP_ORIGSHELL
 	sigconted = 1;
 #  endif
 # endif	/* !MSDOS */
@@ -512,11 +605,11 @@ VOID setlinecol(VOID_A)
 	char buf[MAXLONGWIDTH + 1];
 
 	if (getconstvar(ENVLINES)) {
-		snprintf2(buf, sizeof(buf), "%d", n_line);
+		VOID_C Xsnprintf(buf, sizeof(buf), "%d", n_line);
 		setenv2(ENVLINES, buf, 1);
 	}
 	if (getconstvar(ENVCOLUMNS)) {
-		snprintf2(buf, sizeof(buf), "%d", n_column);
+		VOID_C Xsnprintf(buf, sizeof(buf), "%d", n_column);
 		setenv2(ENVCOLUMNS, buf, 1);
 	}
 }
@@ -557,14 +650,16 @@ int n, max;
 VOID checkscreen(xmax, ymax)
 int xmax, ymax;
 {
-	char *cp, *tty;
+	CONST char *cp;
+	char *tty;
 	int i, row, wastty, dupn_line, dupdumbterm;
 
+	if (ttyio < 0) return;
 #ifdef	SIGWINCH
 	nowinch++;
 #endif
 	dupn_line = -1;
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 	if (parentfd >= 0) row = WFILEMIN;
 	else
 #endif
@@ -591,7 +686,7 @@ int xmax, ymax;
 	for (i = 0;; i++) {
 		if (!(cp = getwsize(xmax, ymax))) {
 			if (n_line >= row) break;
-			cp = NOROW_K;
+			cp = "Line size is too small for operation.";
 		}
 		if (!i) {
 			Xputterms(T_CLEAR);
@@ -599,24 +694,21 @@ int xmax, ymax;
 			keyflush();
 		}
 
-		if (i & 1) Xcputs2(SCRSZ_K);
-		else {
-			Xputterm(T_STANDOUT);
-			Xcputs2(cp);
-			Xputterm(END_STANDOUT);
-		}
+		if (i & 1)
+			cp = "Terminate with the ESC key, or widen a screen.";
+		Xattrputs(cp, !(i & 1));
 		Xputterm(T_BELL);
 		Xcputnl();
 		Xtflush();
-		if (kbhit2(1000000L) && getkey3(0, inputkcode) == K_ESC) {
+		if (kbhit2(1000000L) && getkey3(0, inputkcode, 0) == K_ESC) {
 			errno = 0;
-			error(INTR_K);
+			error("Interrupted by the ESC key.");
 		}
 	}
 	dumbterm = dupdumbterm;
 	loadtermio(ttyio, tty, NULL);
 	isttyiomode = wastty;
-	if (tty) free(tty);
+	Xfree(tty);
 
 	setlinecol();
 	if (n_line != dupn_line) {
@@ -651,21 +743,22 @@ int forced;
 		x = n_column;
 		y = n_line;
 		checkscreen(-1, -1);
-		if (isorgpid()) {
+		if (!shellmode && isorgpid()) {
 			if (x != n_column || y != n_line) rewritefile(1);
-			if (subwindow) ungetch2(K_CTRL('L'));
+			if (subwindow) VOID_C ungetkey2(K_CTRL('L'), 1);
 		}
 	}
 }
 
 static int wintr(VOID_A)
 {
+	sigblk_t old;
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGWINCH, SIG_IGN);
+	old = suspendsignal(SIGWINCH, (sigcst_t)wintr);
 	pollscreen(1);
-	signal2(SIGWINCH, (sigcst_t)wintr);
+	resumesignal(SIGWINCH, old);
 	errno = duperrno;
 
 	return(0);
@@ -681,11 +774,11 @@ int hide;
 
 	if (timersec) now++;
 	else {
-		now = time2();
+		now = Xtime(NULL);
 		timersec = CLOCKUPDATE;
 	}
 	if (timersec-- < CLOCKUPDATE && !showsecond) return;
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 	if (checkallpty() < 0) rewritefile(0);
 	if (isptymode() && !ptyinternal && hide <= 1) hide = 0;
 #endif
@@ -706,11 +799,11 @@ int hide;
 	if (!isleftshift()) x--;
 	Xlocate(x, L_TITLE);
 	Xputterm(T_STANDOUT);
-	Xcprintf2("%02d-%02d-%02d %02d:%02d",
+	VOID_C XXcprintf("%02d-%02d-%02d %02d:%02d",
 		tm -> tm_year % 100,
 		tm -> tm_mon + 1, tm -> tm_mday,
 		tm -> tm_hour, tm -> tm_min);
-	if (showsecond) Xcprintf2(":%02d", tm -> tm_sec);
+	if (showsecond) VOID_C XXcprintf(":%02d", tm -> tm_sec);
 	Xputterm(END_STANDOUT);
 	Xlocate(win_x, win_y);
 	Xtflush();
@@ -719,11 +812,12 @@ int hide;
 #ifdef	SIGALRM
 static int trapalrm(VOID_A)
 {
+	sigblk_t old;
 	int duperrno;
 
 	duperrno = errno;
-	signal2(SIGALRM, SIG_IGN);
-# if	!defined (_NOORIGSHELL) && !defined (_NOPTY)
+	old = suspendsignal(SIGALRM, (sigcst_t)trapalrm);
+# if	defined (DEP_ORIGSHELL) && defined (DEP_PTY)
 	if (isshptymode()) {
 		if (mypid == shellpid) printtime(hideclock);
 	}
@@ -733,19 +827,26 @@ static int trapalrm(VOID_A)
 # ifdef	SIGWINCH
 	pollscreen(0);
 # endif
-	signal2(SIGALRM, (sigcst_t)trapalrm);
+	resumesignal(SIGALRM, old);
 	errno = duperrno;
 
 	return(0);
 }
 #endif	/* SIGALRM */
 
-#ifndef	_NODOSDRIVE
-static int wrap_intrkey(VOID_A)
+#ifdef	DEP_DOSDRIVE
+static int dos_intrkey(VOID_A)
 {
 	return(intrkey(-1));
 }
-#endif	/* !_NODOSDRIVE */
+#endif
+
+static int read_intrkey(VOID_A)
+{
+	if (isttyiomode) return(intrkey(K_ESC));
+
+	return(0);
+}
 
 int sigvecset(set)
 int set;
@@ -757,33 +858,39 @@ int set;
 	if (set == old) /*EMPTY*/;
 	else if (set) {
 #ifdef	SIGALRM
-		signal2(SIGALRM, (sigcst_t)trapalrm);
+		VOID_C signal2(SIGALRM, (sigcst_t)trapalrm);
 		noalrm = 0;
 #endif
 #ifdef	SIGTSTP
-		signal2(SIGTSTP, SIG_IGN);
+		VOID_C signal2(SIGTSTP, SIG_IGN);
 #endif
 #ifdef	SIGWINCH
-		signal2(SIGWINCH, (sigcst_t)wintr);
+		VOID_C signal2(SIGWINCH, (sigcst_t)wintr);
 #endif
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 		doswaitfunc = waitmes;
-		dosintrfunc = wrap_intrkey;
+		dosintrfunc = dos_intrkey;
+#endif
+#ifdef	DEP_LSPARSE
+		lsintrfunc = read_intrkey;
+#endif
+#ifndef	NOSELECT
+		readintrfunc = read_intrkey;
 #endif
 		status = 1;
 	}
 	else {
 #ifdef	SIGALRM
-		signal2(SIGALRM, (sigcst_t)ignore_alrm);
+		VOID_C signal2(SIGALRM, (sigcst_t)ignore_alrm);
 		noalrm = 1;
 #endif
 #ifdef	SIGTSTP
-		signal2(SIGTSTP, SIG_DFL);
+		VOID_C signal2(SIGTSTP, SIG_DFL);
 #endif
 #ifdef	SIGWINCH
-		signal2(SIGWINCH, (sigcst_t)ignore_winch);
+		VOID_C signal2(SIGWINCH, (sigcst_t)ignore_winch);
 #endif
-#ifndef	_NODOSDRIVE
+#ifdef	DEP_DOSDRIVE
 		doswaitfunc = NULL;
 		dosintrfunc = NULL;
 #endif
@@ -793,15 +900,15 @@ int set;
 	return(old);
 }
 
-static char *NEAR getversion(lenp)
+char *getversion(lenp)
 int *lenp;
 {
 	char *cp, *eol;
 
-	cp = strchr(version, ' ');
+	cp = Xstrchr(version, ' ');
 	while (*(++cp) == ' ');
 	if (lenp) {
-		if (!(eol = strchr(cp, ' '))) eol = cp + strlen(cp);
+		if (!(eol = Xstrchr(cp, ' '))) eol = cp + strlen(cp);
 		*lenp = eol - cp;
 	}
 
@@ -810,34 +917,34 @@ int *lenp;
 
 VOID title(VOID_A)
 {
-	char *cp;
-	int i, len;
+	CONST char *cp;
+	int n, len;
 
 	Xlocate(0, L_TITLE);
 	Xputterm(T_STANDOUT);
 	len = 0;
 	if (!isleftshift()) {
-		Xputch2(' ');
+		VOID_C XXputch(' ');
 		len++;
 	}
-	Xcputs2(" FD");
+	XXcputs(" FD");
 	len += 3;
 	if (!ishardomit()) {
-		Xcputs2("(File & Directory tool)");
+		XXcputs("(File & Directory tool)");
 		len += 23;
 	}
-	Xcputs2(" Ver.");
+	XXcputs(" Ver.");
 	len += 5;
-	cp = getversion(&i);
-	cputstr(i, cp);
+	cp = getversion(&n);
+	cputstr(n, cp);
 	if (distributor) {
-		Xputch2('#');
-		i++;
+		VOID_C XXputch('#');
+		n++;
 	}
-	cp = (iswellomit()) ? nullstr : " (c)1995-2007 T.Shirai  ";
-	Xcputs2(cp);
-	i = n_column - len - strlen2(cp) - i;
-	while (i-- > 0) Xputch2(' ');
+	cp = (iswellomit()) ? nullstr : " (c)1995-2017 T.Shirai  ";
+	XXcputs(cp);
+	n = n_column - len - strlen2(cp) - n;
+	while (n-- > 0) VOID_C XXputch(' ');
 	Xputterm(END_STANDOUT);
 	timersec = 0;
 	printtime(0);
@@ -846,26 +953,40 @@ VOID title(VOID_A)
 #ifndef	_NOCUSTOMIZE
 VOID saveorigenviron(VOID_A)
 {
-	orighelpindex = copystrarray(NULL, helpindex, NULL, MAXHELPINDEX);
-	origbindlist = copybind(NULL, bindlist);
+# if	FD >= 3
+	char *cp;
+	int n;
+# endif
+
 # ifndef	_NOKEYMAP
 	origkeymaplist = copykeyseq(NULL);
 # endif
-# ifndef	_NOARCHIVE
+# ifndef	DEP_DYNAMICLIST
+	orighelpindex = copystrarray(NULL, helpindex, NULL, MAXHELPINDEX);
+	origbindlist = copybind(NULL, bindlist, &origmaxbind, maxbind);
+#  ifndef	_NOARCHIVE
 	origlaunchlist = copylaunch(NULL, launchlist,
 		&origmaxlaunch, maxlaunch);
 	origarchivelist = copyarch(NULL, archivelist,
 		&origmaxarchive, maxarchive);
-# endif
-# ifdef	_USEDOSEMU
-	origfdtype = copydosdrive(NULL, fdtype);
+#  endif
+#  ifdef	DEP_DOSEMU
+	origfdtype = copydosdrive(NULL, fdtype, &origmaxfdtype, maxfdtype);
+#  endif
+# endif	/* !DEP_DYNAMICLIST */
+
+# if	FD >= 3
+	cp = getversion(&n);
+	cp = Xstrndup(cp, n);
+	setenv2(FDVERSION, cp, 0);
+	Xfree(cp);
 # endif
 }
 #endif	/* !_NOCUSTOMIZE */
 
-#ifndef	_NOLOGGING
+#ifdef	DEP_LOGGING
 static VOID NEAR startlog(argv)
-char **argv;
+char *CONST *argv;
 {
 # ifndef	NOUID
 	uid_t uid;
@@ -878,9 +999,12 @@ char **argv;
 	for (i = 1; argv && argv[i]; i++) {
 		if (cp > buf) *(cp++) = ' ';
 		tmp = killmeta(argv[i]);
-		len = snprintf2(cp, (int)sizeof(buf) - (cp - buf), "%s", tmp);
-		free(tmp);
+		len = Xsnprintf(cp, (int)sizeof(buf) - (cp - buf), "%s", tmp);
+		Xfree(tmp);
 		if (len < 0) break;
+#ifdef	CODEEUC
+		len = strlen(cp);
+#endif
 		cp += len;
 	}
 	*cp = '\0';
@@ -892,7 +1016,7 @@ char **argv;
 # else
 	lvl = ((uid = getuid())) ? _LOG_DEBUG_ : _LOG_WARNING_;
 	logmessage(lvl, "%s (%-*.*s) starts; UID=%d; PWD=%k; ARGS=%k",
-		progname, len, len, cp, uid, origpath, buf);
+		progname, len, len, cp, (int)uid, origpath, buf);
 # endif
 }
 
@@ -905,24 +1029,25 @@ int status;
 # endif
 	char cwd[MAXPATHLEN];
 
-	if (!Xgetwd(cwd)) strcpy(cwd, "?");
+	if (!Xgetwd(cwd)) Xstrcpy(cwd, "?");
 # ifdef	NOUID
 	logmessage(_LOG_DEBUG_, "%s ends; PWD=%k; STATUS=%d",
 		progname, cwd, status);
 # else
 	lvl = ((uid = getuid())) ? _LOG_DEBUG_ : _LOG_WARNING_;
 	logmessage(lvl, "%s ends; UID=%d; PWD=%k; STATUS=%d",
-		progname, uid, cwd, status);
+		progname, (int)uid, cwd, status);
 # endif
 	logclose();
 }
-#endif	/* !_NOLOGGING */
+#endif	/* DEP_LOGGING */
 
-#ifdef	_NOORIGSHELL
+#ifndef	DEP_ORIGSHELL
 static int NEAR execruncomline(command, file, n, line)
-char *command, *file;
+char *command;
+CONST char *file;
 int n;
-char *line;
+CONST char *line;
 {
 	char *cp;
 	int i;
@@ -931,75 +1056,80 @@ char *line;
 	else i = execpseudoshell(cp, F_IGNORELIST | F_NOCOMLINE);
 	if (i) {
 		Xputterm(L_CLEAR);
-		Xcprintf2("%s, line %d: %s", file, n, ILFNC_K);
+		VOID_C XXcprintf("%s, line %d: %k", file, n, ILFNC_K);
 		Xcputnl();
 		Xputterm(L_CLEAR);
-		Xcprintf2("\t%s", line);
+		VOID_C XXcprintf("\t%s", line);
 		Xcputnl();
 	}
-	free(command);
+	Xfree(command);
 
 	return((i) ? -1 : 0);
 }
-#endif	/* _NOORIGSHELL */
+#endif	/* !DEP_ORIGSHELL */
 
 /*ARGSUSED*/
 int loadruncom(file, exist)
-char *file;
+CONST char *file;
 int exist;
 {
-#ifdef	_NOORIGSHELL
-	FILE *fp;
+#ifndef	DEP_ORIGSHELL
+# if	!MSDOS
+	struct stat st;
+# endif
+	XFILE *fp;
 	char *cp, *tmp, *fold, *line;
 	int cont;
-#endif
+#endif	/* !DEP_ORIGSHELL */
 	int n, er;
 
-#ifdef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
+	if ((n = isttyiomode)) Xstdiomode();
+	er = execruncom(file, 1);
+	if (n) Xttyiomode(n - 1);
+#else	/* !DEP_ORIGSHELL */
 # if	!MSDOS
 	tmp = NULL;
 	if (!exist && (tmp = getconstvar(ENVTERM))) {
-		struct stat st;
-
-		cp = malloc2(strlen(file) + strlen(tmp) + 1 + 1);
-		strcpy(strcpy2(strcpy2(cp, file), "."), tmp);
+		cp = Xmalloc(strlen(file) + strlen(tmp) + 1 + 1);
+		Xstrcpy(Xstrcpy(Xstrcpy(cp, file), "."), tmp);
 		tmp = evalpath(cp, 0);
 		if (stat2(tmp, &st) < 0 || !s_isreg(&st)) {
-			free(tmp);
+			Xfree(tmp);
 			tmp = NULL;
 		}
 	}
 	if (!tmp)
 # endif	/* !MSDOS */
-	tmp = evalpath(strdup2(file), 0);
+	tmp = evalpath(Xstrdup(file), 0);
 	fp = Xfopen(tmp, "r");
 	if (!fp) {
 		if (!exist) {
-			free(tmp);
+			Xfree(tmp);
 			return(0);
 		}
-		Xcprintf2("%s: Not found", tmp);
+		VOID_C XXcprintf("%s: Not found", tmp);
 		Xcputnl();
-		free(tmp);
+		Xfree(tmp);
 		return(-1);
 	}
 
 	fold = NULL;
 	n = er = 0;
-	while ((line = fgets2(fp, 0))) {
+	while ((line = Xfgets(fp))) {
 		n++;
 		if (*line == ';' || *line == '#') {
-			free(line);
+			Xfree(line);
 			continue;
 		}
 
 		cp = line + strlen(line);
-		for (cp--; cp >= line && isblank2(*cp); cp--);
+		for (cp--; cp >= line; cp--) if (!Xisblank(*cp)) break;
 		cp[1] = '\0';
 
 		cont = 0;
-		if (cp >= line && *cp == META
-		&& (cp - 1 < line || *(cp - 1) != META)
+		if (cp >= line && *cp == ESCAPE
+		&& (cp - 1 < line || *(cp - 1) != ESCAPE)
 		&& !onkanji1(line, cp - line - 1)) {
 			*cp = '\0';
 			cont = 1;
@@ -1007,42 +1137,37 @@ int exist;
 
 		if (!fold) fold = line;
 		else if (*line) {
-			fold = realloc2(fold, strlen(fold) + strlen(line) + 1);
+			fold = Xrealloc(fold, strlen(fold) + strlen(line) + 1);
 			strcat(fold, line);
 		}
 
 		if (cont) {
-			if (fold != line) free(line);
+			if (fold != line) Xfree(line);
 			continue;
 		}
 
 		if (execruncomline(fold, tmp, n, line) < 0) er++;
-		if (fold != line) free(line);
+		if (fold != line) Xfree(line);
 		fold = NULL;
 	}
 
 	if (fold && execruncomline(fold, tmp, n, line) < 0) er++;
-	fclose(fp);
-	free(tmp);
-#else	/* !_NOORIGSHELL */
-	if ((n = isttyiomode)) Xstdiomode();
-	er = execruncom(file, 1);
-	if (n) Xttyiomode(n - 1);
-#endif	/* !_NOORIGSHELL */
+	VOID_C Xfclose(fp);
+	Xfree(tmp);
+#endif	/* !DEP_ORIGSHELL */
 
 	return(er ? -1 : 0);
 }
 
-/*ARGSUSED*/
-static int NEAR initoption(argc, argv, envp)
+static int NEAR initoption(argc, argv)
 int argc;
-char *argv[], *envp[];
+char *CONST argv[];
 {
 	char *cp, **optv;
 	int i, optc;
 
 	optc = 1;
-	optv = (char **)malloc2((argc + 1) * sizeof(char *));
+	optv = (char **)Xmalloc((argc + 1) * sizeof(char *));
 	optv[0] = argv[0];
 
 	for (i = 1; i < argc; i++) {
@@ -1064,16 +1189,16 @@ char *argv[], *envp[];
 	}
 	optv[optc] = NULL;
 
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 	if (initshell(optc, optv) < 0) Xexit2(RET_FAIL);
-#endif	/* !_NOORIGSHELL */
-	free(optv);
+#endif
+	Xfree(optv);
 
 	return(argc);
 }
 
 static int NEAR evaloption(argv)
-char *argv[];
+char *CONST argv[];
 {
 	char *cp, *tmp;
 	int i;
@@ -1084,17 +1209,18 @@ char *argv[];
 			i++;
 			break;
 		}
-		tmp = strdup2(&(argv[i][1]));
-		if ((cp = strchr(tmp, '='))) *(cp++) = '\0';
+		tmp = Xstrdup(&(argv[i][1]));
+		if ((cp = Xstrchr(tmp, '='))) *(cp++) = '\0';
 		setenv2(tmp, cp, 0);
-		free(tmp);
+		Xfree(tmp);
 	}
 
 	return(i);
 }
 
 static char *NEAR searchenv(s, envp)
-char *s, *envp[];
+CONST char *s;
+char *CONST envp[];
 {
 	int i, len;
 
@@ -1106,8 +1232,40 @@ char *s, *envp[];
 	return(NULL);
 }
 
+#if	!MSDOS
+static char *NEAR searchexecname(argv, envp)
+CONST char *argv;
+char *CONST envp[];
+{
+	CONST char *s, *env;
+	char *cp;
+	int n;
+
+	n = 0;
+	if (!(env = searchenv(ENVPATH, envp))) env = DEFPATH;
+	s = progname;
+	if (*s == '-') s++;
+	if (strpathcmp(s, SWITCHUSER)) {
+		if ((cp = searchexecpath(argv, env))) return(cp);
+		if (*argv == '-') {
+			if ((cp = searchexecpath(++argv, env))) return(cp);
+			n++;
+		}
+		if (*argv == 'r' && (cp = searchexecpath(++argv, env)))
+			return(cp);
+		if ((cp = searchexecpath(FDSHELL, env))) return(cp);
+		if ((cp = searchexecpath(FDPROG, env))) return(cp);
+
+		if (!n) return(NULL);
+	}
+
+	getlogininfo(NULL, &s);
+	return(Xstrdup(s));
+}
+#endif	/* !MSDOS */
+
 static VOID NEAR setexecname(argv)
-char *argv;
+CONST char *argv;
 {
 #if	MSDOS || defined (CYGWIN)
 	char *cp;
@@ -1115,55 +1273,60 @@ char *argv;
 
 	progname = getbasename(argv);
 #if	MSDOS || defined (CYGWIN)
-	if ((cp = strchr(progname, '.')) && cp > progname)
-		progname = strndup2(progname, cp - progname);
+	if ((cp = Xstrchr(progname, '.')) && cp > progname)
+		progname = Xstrndup(progname, cp - progname);
 	else
 #endif
-	progname = strdup2(progname);
+	progname = Xstrdup(progname);
 }
 
 static VOID NEAR setexecpath(argv, envp)
-char *argv, *envp[];
+CONST char *argv;
+char *CONST envp[];
 {
-	char *cp, buf[MAXPATHLEN];
+	CONST char *cp;
+	char *tmp, buf[MAXPATHLEN];
 
-	if (!Xgetwd(buf)) error(NOCWD_K);
-	origpath = strdup2(buf);
+	if (!Xgetwd(buf)) {
+		errno = 0;
+		error("No current working directory.");
+	}
+	origpath = Xstrdup(buf);
 	if ((cp = searchenv(ENVPWD, envp))) {
 		*fullpath = '\0';
-		realpath2(cp, fullpath, 0);
-		realpath2(fullpath, buf, 1);
+		Xrealpath(cp, fullpath, 0);
+		Xrealpath(fullpath, buf, RLP_READLINK);
 		if (!strpathcmp(origpath, buf)) {
-			free(origpath);
-			origpath = strdup2(fullpath);
+			Xfree(origpath);
+			origpath = Xstrdup(fullpath);
 		}
 	}
-	strcpy(fullpath, origpath);
+	Xstrcpy(fullpath, origpath);
 
+	tmp = NULL;
 #if	MSDOS
 	cp = argv;
 #else
 	if (strdelim(argv, 0)) cp = argv;
-	else if ((cp = searchenv(ENVPATH, envp)))
-		cp = searchexecpath(argv, cp);
-	if (!cp) progpath = strdup2(origpath);
+	else cp = tmp = searchexecname(argv, envp);
+	if (!cp) progpath = Xstrdup(BINDIR);
 	else
 #endif
 	{
-		realpath2(cp, buf, 1);
-		if (cp != argv) free(cp);
-		if ((cp = strrdelim(buf, 0))) *cp = '\0';
-		progpath = strdup2(buf);
+		Xrealpath(cp, buf, RLP_READLINK);
+		Xfree(tmp);
+		if ((tmp = strrdelim(buf, 0))) *tmp = '\0';
+		progpath = Xstrdup(buf);
 	}
 }
 
 /*ARGSUSED*/
 VOID initfd(argv)
-char **argv;
+char *CONST *argv;
 {
 	int i;
 
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 	if (!interactive) /*EMPTY*/;
 	else
 #endif
@@ -1176,9 +1339,9 @@ char **argv;
 		}
 #endif	/* !MSDOS */
 		for (i = 0; i < 2; i++) loadhistory(i);
-		entryhist(1, origpath, 1);
+		VOID_C entryhist(origpath, HST_PATH | HST_UNIQ);
 	}
-#ifndef	_NOLOGGING
+#ifdef	DEP_LOGGING
 	startlog(argv);
 #endif
 }
@@ -1190,10 +1353,10 @@ int status;
 	char cwd[MAXPATHLEN];
 	int i;
 
-#ifndef	_NOLOGGING
+#ifdef	DEP_LOGGING
 	endlog(status);
 #endif
-#ifndef	_NOORIGSHELL
+#ifdef	DEP_ORIGSHELL
 	if (!interactive) /*EMPTY*/;
 	else
 #endif
@@ -1203,52 +1366,53 @@ int status;
 	if (origpath && _chdir2(origpath) < 0) {
 		perror2(origpath);
 		if (!Xgetwd(cwd)) *cwd = '\0';
-		rawchdir(rootpath);
+		VOID_C rawchdir(rootpath);
 	}
-	free(origpath);
-	free(progname);
-#ifdef	_USEUNICODE
-	free(unitblpath);
+	Xfree(origpath);
+	Xfree(progname);
+#ifdef	DEP_UNICODE
+	Xfree(unitblpath);
 #endif
-#ifndef	_NOIME
-	free(dicttblpath);
+#ifdef	DEP_IME
+	Xfree(dicttblpath);
 #endif
-#ifndef	_NODOSDRIVE
-	dosallclose();
+#ifndef	_NOCATALOG
+	Xfree(cattblpath);
 #endif
-	free(progpath);
+#ifdef	DEP_DOSDRIVE
+	VOID_C dosallclose();
+#endif
+#ifdef	DEP_URLPATH
+	VOID_C urlallclose();
+#endif
+	Xfree(progpath);
 
 #ifdef	DEBUG
-	if (tmpfilename) free(tmpfilename);
-# ifdef	_NOORIGSHELL
+	Xfree(tmpfilename);
+	tmpfilename = NULL;
+# ifndef	DEP_ORIGSHELL
 	freevar(environ);
 	freevar(environ2);
 # endif
 # ifndef	_NOCUSTOMIZE
-	if (orighelpindex) {
-		freestrarray(orighelpindex, MAXHELPINDEX);
-		free(orighelpindex);
-	}
-	free(origbindlist);
 #  ifndef	_NOKEYMAP
 	freekeyseq(origkeymaplist);
 #  endif
-#  ifndef	_NOARCHIVE
-	if (origlaunchlist) {
-		freelaunchlist(origlaunchlist, origmaxlaunch);
-		free(origlaunchlist);
-	}
-	if (origarchivelist) {
-		freearchlist(origarchivelist, origmaxarchive);
-		free(origarchivelist);
-	}
-#  endif
-#  ifdef	_USEDOSEMU
-	if (origfdtype) {
-		freedosdrive(origfdtype);
-		free(origfdtype);
-	}
-#  endif
+#  ifndef	DEP_DYNAMICLIST
+	freestrarray(orighelpindex, MAXHELPINDEX);
+	Xfree(orighelpindex);
+	Xfree(origbindlist);
+#   ifndef	_NOARCHIVE
+	freelaunchlist(origlaunchlist, origmaxlaunch);
+	Xfree(origlaunchlist);
+	freearchlist(origarchivelist, origmaxarchive);
+	Xfree(origarchivelist);
+#   endif
+#   ifdef	DEP_DOSEMU
+	freedosdrive(origfdtype, origmaxfdtype);
+	Xfree(origfdtype);
+#   endif
+#  endif	/* !DEP_DYNAMICLIST */
 # endif	/* !_NOCUSTOMIZE */
 	freeenvpath();
 	freehistory(0);
@@ -1258,31 +1422,37 @@ int status;
 	freeidlist();
 # endif
 	chdir2(NULL);
-# if	!defined (_NOUSEHASH) && defined (_NOORIGSHELL)
-	searchhash(NULL, NULL, NULL);
+# if	!defined (_NOUSEHASH) && !defined (DEP_ORIGSHELL)
+	VOID_C searchhash(NULL, NULL, NULL);
 # endif
 # ifndef	_NOROCKRIDGE
 	detranspath(NULL, NULL);
 # endif
-# ifdef	_USEUNICODE
+# ifdef	DEP_UNICODE
 	discardunitable();
 # endif
-# ifndef	_NOIME
+# ifdef	DEP_IME
 	ime_freebuf();
 	freeroman(0);
 	discarddicttable();
 # endif
 #endif	/* DEBUG */
-	if (*cwd) rawchdir(cwd);
+#ifndef	_NOCATALOG
+	freecatalog();
+#endif
+	if (*cwd) VOID_C rawchdir(cwd);
 }
 
 int main(argc, argv, envp)
 int argc;
-char *argv[], *envp[];
+char *CONST argv[], *CONST envp[];
 {
 	char *cp;
 	int i;
 
+#ifndef	NOFLOCK
+	stream_isnfsfunc = isnfs;
+#endif
 #ifdef	DEBUG
 	mtrace();
 #endif
@@ -1296,135 +1466,100 @@ char *argv[], *envp[];
 	_harderr(criticalerror);
 #endif
 
-#ifdef	SIGHUP
-	signal2(SIGHUP, (sigcst_t)hanguperror);
-#endif
-#ifdef	SIGINT
-	signal2(SIGINT, (sigcst_t)ignore_int);
-#endif
-#ifdef	SIGQUIT
-	signal2(SIGQUIT, (sigcst_t)ignore_quit);
-#endif
-#ifdef	SIGCONT
-	signal2(SIGCONT, (sigcst_t)ignore_cont);
-#endif
-#ifdef	SIGILL
-	signal2(SIGILL, (sigcst_t)illerror);
-#endif
-#ifdef	SIGTRAP
-	signal2(SIGTRAP, (sigcst_t)traperror);
-#endif
-#ifdef	SIGIOT
-	signal2(SIGIOT, (sigcst_t)ioerror);
-#endif
-#ifdef	SIGEMT
-	signal2(SIGEMT, (sigcst_t)emuerror);
-#endif
-#ifdef	SIGFPE
-	signal2(SIGFPE, (sigcst_t)floaterror);
-#endif
-#ifdef	SIGBUS
-	signal2(SIGBUS, (sigcst_t)buserror);
-#endif
-#ifdef	SIGSEGV
-	signal2(SIGSEGV, (sigcst_t)segerror);
-#endif
-#ifdef	SIGSYS
-	signal2(SIGSYS, (sigcst_t)syserror);
-#endif
-#ifdef	SIGPIPE
-	signal2(SIGPIPE, (sigcst_t)pipeerror);
-#endif
-#ifdef	SIGTERM
-	signal2(SIGTERM, (sigcst_t)terminate);
-#endif
-#ifdef	SIGXCPU
-	signal2(SIGXCPU, (sigcst_t)xcpuerror);
-#endif
-#ifdef	SIGXFSZ
-	signal2(SIGXFSZ, (sigcst_t)xsizerror);
-#endif
+	for (i = 0; i < arraysize(sigfunclist); i++) {
+		VOID_C signal2(sigfunclist[i].sig,
+			(sigcst_t)(sigfunclist[i].func));
+	}
 	sigvecset(0);
 
-#ifndef	_NOARCHIVE
-	for (maxlaunch = 0; maxlaunch < MAXLAUNCHTABLE; maxlaunch++) {
-		if (!launchlist[maxlaunch].ext) break;
-		launchlist[maxlaunch].ext =
-			strdup2(launchlist[maxlaunch].ext);
-		launchlist[maxlaunch].comm =
-			strdup2(launchlist[maxlaunch].comm);
-# if	FD >= 2
-		launchlist[maxlaunch].format =
-			duplvar(launchlist[maxlaunch].format, -1);
-		launchlist[maxlaunch].lignore =
-			duplvar(launchlist[maxlaunch].lignore, -1);
-		launchlist[maxlaunch].lerror =
-			duplvar(launchlist[maxlaunch].lerror, -1);
+	VOID_C entryhist(NULL, HST_INIT);
+#ifdef	DEP_DYNAMICLIST
+	helpindex = copystrarray(NULL, orighelpindex, NULL, MAXHELPINDEX);
+	for (i = 0; origbindlist[i].key >= 0; i++) /*EMPTY*/;
+	origmaxbind = i;
+	bindlist = copybind(NULL, origbindlist, &maxbind, origmaxbind);
+# ifndef	_NOARCHIVE
+	for (i = 0; origlaunchlist[i].ext; i++) /*EMPTY*/;
+	origmaxlaunch = i;
+	launchlist = copylaunch(NULL, origlaunchlist,
+		&maxlaunch, origmaxlaunch);
+	for (i = 0; origarchivelist[i].ext; i++) /*EMPTY*/;
+	origmaxarchive = i;
+	archivelist = copyarch(NULL, origarchivelist,
+		&maxarchive, origmaxarchive);
 # endif
-	}
-	for (maxarchive = 0; maxarchive < MAXARCHIVETABLE; maxarchive++) {
-		if (!archivelist[maxarchive].ext) break;
-		archivelist[maxarchive].ext =
-			strdup2(archivelist[maxarchive].ext);
-		archivelist[maxarchive].p_comm =
-			strdup2(archivelist[maxarchive].p_comm);
-		archivelist[maxarchive].u_comm =
-			strdup2(archivelist[maxarchive].u_comm);
-	}
-#endif	/* !_NOARCHIVE */
-
-	for (i = 0; i < MAXHELPINDEX; i++)
-		helpindex[i] = strdup2(helpindex[i]);
-#ifdef	_USEDOSEMU
-	for (i = 0; fdtype[i].name; i++) {
-		fdtype[i].name = strdup2(fdtype[i].name);
-# ifdef	HDDMOUNT
-		fdtype[i].offset = 0;
+# ifdef	DEP_DOSEMU
+	for (i = 0; origfdtype[i].name; i++) /*EMPTY*/;
+	origmaxfdtype = i;
+	fdtype = copydosdrive(NULL, origfdtype, &maxfdtype, origmaxfdtype);
 # endif
-	}
-#endif
+#else	/* !DEP_DYNAMICLIST */
+	copystrarray(helpindex, helpindex, NULL, MAXHELPINDEX);
+	for (i = 0; i < MAXBINDTABLE && bindlist[i].key >= 0; i++) /*EMPTY*/;
+	maxbind = i;
+# ifndef	_NOARCHIVE
+	for (i = 0; i < MAXLAUNCHTABLE && launchlist[i].ext; i++) /*EMPTY*/;
+	maxlaunch = i;
+	copylaunch(launchlist, launchlist, &maxlaunch, maxlaunch);
+	for (i = 0; i < MAXARCHIVETABLE && archivelist[i].ext; i++) /*EMPTY*/;
+	maxarchive = i;
+	copyarch(archivelist, archivelist, &maxarchive, maxarchive);
+# endif
+# ifdef	DEP_DOSEMU
+	for (i = 0; i < MAXDRIVEENTRY && fdtype[i].name; i++) /*EMPTY*/;
+	maxfdtype = i;
+	copydosdrive(fdtype, fdtype, &maxfdtype, maxfdtype);
+# endif
+#endif	/* !DEP_DYNAMICLIST */
 
-#ifdef	_NOORIGSHELL
-	for (i = 0; envp[i]; i++);
-	environ = (char **)malloc2((i + 1) * sizeof(char *));
-	for (i = 0; envp[i]; i++) environ[i] = strdup2(envp[i]);
+#ifndef	DEP_ORIGSHELL
+	i = countvar(envp);
+	environ = (char **)Xmalloc((i + 1) * sizeof(char *));
+	for (i = 0; envp[i]; i++) environ[i] = Xstrdup(envp[i]);
 	environ[i] = NULL;
 #endif
 
 	setexecpath(argv[0], envp);
-#ifdef	_USEUNICODE
-	unitblpath = strdup2(DATADIR);
+#ifdef	DEP_UNICODE
+	unitblpath = Xstrdup(DATADIR);
 #endif
-#ifndef	_NOIME
-	dicttblpath = strdup2(DATADIR);
+#ifdef	DEP_IME
+	dicttblpath = Xstrdup(DATADIR);
 #endif
+#ifndef	_NOCATALOG
+# ifdef	USEDATADIR
+	cp = getversion(&i);
+	cattblpath = Xmalloc(strsize(DATADIR) + 1 + i + 1);
+	VOID_C Xsnprintf(cattblpath, strsize(DATADIR) + 1 + i + 1,
+		"%s%c%-.*s", DATADIR, _SC_, i, cp);
+# else
+	cattblpath = Xstrdup(DATADIR);
+# endif
+#endif	/* !_NOCATALOG */
 
-#ifdef	_NOORIGSHELL
-	inittty(0);
-	getterment(NULL);
-	argc = initoption(argc, argv, envp);
-	adjustpath();
-#else	/* !_NOORIGSHELL */
+#ifdef	DEP_ORIGSHELL
 	cp = getshellname(progname, NULL, NULL);
-	if (!strpathcmp(cp, FDSHELL) || !strpathcmp(cp, "su")) {
+	if (!strpathcmp(cp, FDSHELL) || !strpathcmp(cp, SWITCHUSER)) {
 		i = main_shell(argc, argv, envp);
-# ifndef	_NOPTY
+# ifdef	DEP_PTY
 		killallpty();
 # endif
 		Xexit2(i);
 	}
-#ifndef	_NOORIGSHELL
-	interactive =
-#endif
-	fdmode = 1;
+	interactive = fdmode = 1;
 	setshellvar(envp);
-	prepareterm();
-	argc = initoption(argc, argv, envp);
+	if (prepareterm() < 0) error(NULL);
+	argc = initoption(argc, argv);
 	if (dumbterm > 1) {
 		errno = 0;
 		error(NTERM_K);
 	}
-#endif	/* !_NOORIGSHELL */
+#else	/* !DEP_ORIGSHELL */
+	inittty(0);
+	getterment(NULL);
+	argc = initoption(argc, argv);
+	adjustpath();
+#endif	/* !DEP_ORIGSHELL */
 	initenv();
 
 	Xttyiomode(0);
@@ -1460,21 +1595,14 @@ char *argv[], *envp[];
 #ifdef	SIGWINCH
 	nowinch = 0;
 #endif
-#if	FD >= 2
-	main_fd(&(argv[i]), (thruargs) ? 1 : 0);
-#else
 	main_fd(&(argv[i]), 0);
-#endif
 	sigvecset(0);
-#ifndef	_NOLOGGING
-	endlog(0);
-#endif
 
 	Xstdiomode();
-#if	!defined (_NOORIGSHELL) && !defined (NOJOB)
+#if	defined (DEP_ORIGSHELL) && !defined (NOJOB)
 	killjob();
 #endif
-#ifndef	_NOPTY
+#ifdef	DEP_PTY
 	killallpty();
 #endif
 	Xexit2(0);
